import json
from datetime import datetime
from typing import List, Optional

import pydantic


class BaseModel(pydantic.BaseModel):
    @classmethod
    def from_dict(cls, data):
        return cls(**data)

    @classmethod
    def from_json(cls, json_data):
        data = json.loads(json_data)
        return cls.from_dict(data)


class Building(BaseModel):
    id: int
    # area: Area
    external_id: str
    name: str
    acronym: Optional[str]
    url: Optional[str]
    created: datetime
    updated: datetime


class Room(BaseModel):
    id: int
    building: Building
    external_id: str
    name: str
    acronym: Optional[str]
    url: Optional[str]
    created: datetime
    updated: datetime


class EventLocation(BaseModel):
    room: Room


class Event(BaseModel):
    id: int
    meta: dict
    created: datetime
    updated: datetime
    start: datetime
    end: datetime
    source: str
    external_id: str
    course_id: str
    locations: List[EventLocation]
    kind: str


class Person(BaseModel):
    id: int
    uid: str


class Group(BaseModel):
    id: int
    meta: dict | None
    created: datetime
    updated: datetime
    source: str
    external_id: str
    kind: str
    events: List[int]
